#include <iostream>
#include <sstream>
#include <vector>
#include <map>

#include "gtp.h"

class sample_engine :public engine {
  int size;
  float komi;
  struct move {
    int x;
    int y;
    COLOR color;    
    move(int _x, int _y, COLOR _color)
      :x(_x), y(_y), color(_color)
    {}
    ~move(){}
  };
  std::vector<move> moves;
  COLOR board[MAX_BOARD][MAX_BOARD];
  COLOR status[MAX_BOARD][MAX_BOARD];

 public:
  sample_engine()
    :size(19), komi(0), moves()
  {
    reset(size, komi);
  }
  virtual ~sample_engine() {}

  virtual void reset(int size, float komi) {
    this->komi = komi;
    this->size = size;
    for (int i=0; i < MAX_BOARD; i++) {
      for (int j=0; j < MAX_BOARD; j++) {
	board[i][j] = EMPTY;
      }
    }
    moves.clear();
  }

  virtual std::string name() {
    return "sample";
  }
  virtual std::string version() {
    return "0.0";
  }

  virtual void genmove(int& i, int& j, COLOR color){
    for (int ii=0; ii < size; ii++) {
      for (int jj=0; jj < size; jj++) {
	if (board[ii][jj] == EMPTY) {
	  i = ii;
	  j = jj;
	  return;
	}
      }
    }
    // PASS
    i = -1;
    j = -1;
    return;
  }

  virtual void playmove(int i, int j, COLOR color){
    moves.push_back(move(i, j, color));
    board[i][j] = color;
    return;
  }

  virtual void undo(){
    move m = moves.back();
    board[m.x][m.y] = EMPTY;
    moves.pop_back();
  }

  virtual bool legal_move(int i, int j, COLOR color){
    return board[i][j] == EMPTY;
  }

  virtual void compute_final_status() {
    for (int i = 0; i < size; i++) {
      for (int j = 0; j < size; j++) {
	if (board[i][j] == BLACK) {
	  status[i][j] = BLACK_TERRITORY;
	} else if (board[i][j] == WHITE) {
	  status[i][j] = WHITE_TERRITORY;
	} else {
	  status[i][j] = EMPTY;
	}
      }
    }
  }
  
  virtual COLOR get_board(int i, int j) {
    return board[i][j];
  }
  virtual COLOR get_final_status(int i, int j) {
    return status[i][j];
  }

  virtual void time_settings(int main_time,
			     int byo_yomi_time,
			     int byo_yomi_stones) {
  }
  virtual void time_left(COLOR color, int time, int stones) {
  }


};

int
main(void)
{
  gtp g(new sample_engine);
  g.main_loop();
  return 0;
}

